@@ -11,9 +11,9 @@
 */
 
 /*
- * $Id: msg.c 8 2008-05-28 09:27:26Z rmanfredi $
+ * $Id: msg.c,v 3.0.1.3 1995/08/31 16:22:00 ram Exp $
  *
- *  Copyright (c) 1990-2006, Raphael Manfredi
+ *  Copyright (c) 1990-1993, Raphael Manfredi
  *  
  *  You may redistribute only under the terms of the Artistic License,
  *  as specified in the README file that comes with the distribution.
@@ -22,12 +22,6 @@
  *  of the source tree for mailagent 3.0.
  *
  * $Log: msg.c,v $
- * Revision 3.0.1.5  1999/01/13  18:08:06  ram
- * patch64: added tag-checking heuristic to say()
- *
- * Revision 3.0.1.4  1996/12/24  13:59:31  ram
- * patch45: call my_exit() instead of exit()
- *
  * Revision 3.0.1.3  1995/08/31  16:22:00  ram
  * patch42: new routine say() to print messages onto stderr
  * patch42: all messages on stderr now also include the filter pid
@@ -48,7 +42,6 @@
 #include "portable.h"
 #include <stdio.h>
 #include <sys/types.h>
-#include <ctype.h>
 #include "sysexits.h"
 #include "logfile.h"
 #include "lock.h"
@@ -57,7 +50,7 @@
 
 #define MAX_STRING	1024		/* Maximum length for error string */
 
-extern void my_exit();
+extern Pid_t progpid;			/* Program PID */
 
 /* VARARGS2 */
 public void say(msg, arg1, arg2, arg3, arg4, arg5)
@@ -69,21 +62,6 @@ long arg1, arg2, arg3, arg4, arg5;	/* Use longs, hope (char *) fits in it! */
 	fprintf(stderr, "%s[%d]: ", progname, progpid);
 	fprintf(stderr, msg, arg1, arg2, arg3, arg4, arg5);
 	fputc('\n', stderr);
-
-	/*
-	 * A little heuristic here...
-	 *
-	 * If the message begins with an upper-cased don't prepend
-	 * the ERROR tag, assuming a tag was already specified.
-	 */
-
-	if (isupper(msg[0]))
-		add_log(2, msg, arg1, arg2, arg3, arg4, arg5);
-	else {
-		char buffer[MAX_STRING];
-		sprintf(buffer, "ERROR %s", msg);
-		add_log(2, buffer, arg1, arg2, arg3, arg4, arg5);
-	}
 }
 
 /* VARARGS2 */
@@ -107,7 +85,6 @@ long arg1, arg2, arg3, arg4, arg5;	/* Use longs, hope (char *) fits in it! */
 	fprintf(stderr, "%s[%d]: FATAL ", progname, progpid);
 	fprintf(stderr, reason, arg1, arg2, arg3, arg4, arg5);
 	fputc('\n', stderr);
-
 	sprintf(buffer, "FATAL %s", reason);
 	add_log(1, buffer, arg1, arg2, arg3, arg4, arg5);
 
@@ -132,7 +109,7 @@ long arg1, arg2, arg3, arg4, arg5;	/* Use longs, hope (char *) fits in it! */
 		 */
 
 		add_log(6, "NOTICE leaving mail in MTA's queue");
-		my_exit(EX_TEMPFAIL);
+		exit(EX_TEMPFAIL);
 	}
 
 	/*
@@ -141,6 +118,6 @@ long arg1, arg2, arg3, arg4, arg5;	/* Use longs, hope (char *) fits in it! */
 	 * the MTA to worry, hence the following...
 	 */
 
-	my_exit(EX_OK);
+	exit(EX_OK);
 }
 
