@@ -1,6 +1,6 @@
-;# $Id: compress.pl 15 2008-05-29 22:13:02Z rmanfredi $
+;# $Id: compress.pl,v 3.0.1.1 1995/09/15 14:03:35 ram Exp $
 ;#
-;#  Copyright (c) 1990-2006, Raphael Manfredi
+;#  Copyright (c) 1990-1993, Raphael Manfredi
 ;#  
 ;#  You may redistribute only under the terms of the Artistic License,
 ;#  as specified in the README file that comes with the distribution.
@@ -54,20 +54,16 @@ sub init {
 		next if /^\s*$/;			# And blank lines
 		$_ = &'perl_pattern($_);	# Shell pattern to perl one
 		s/^~/$cf'home/;				# ~ substitution
-		# Focus on basename unless absolute path
-		$_ = '(?>.*/)'.$_ unless m|^/|;
+		$_ = '.*/'.$_ unless m|^/|;	# Focus on basename unless absolute path
 		push(@compress, $_);		# Record pattern
 	}
 	close COMPRESS;
 
 	unless (open(COMPSPEC, "$cf'compspec")) {
 		# Configure a set of defaults if the user hasn't specified them manually
-		# Fields are: tag extension compression_prog uncompress_prog cat_prog
-		# The following legacy line removed as modern systems lack compress:
-		# compress	.Z	compress	uncompress	zcat
+#compress	.Z	compress	uncompress	zcat
 		&add_compressor(<<'EOT');
-gzip		.gz		gzip		gunzip		gunzip -c
-bzip2		.bz2	bzip2		bunzip2		bzcat
+gzip		.gz	gzip		gunzip		gunzip -c
 EOT
 		local($err) = "$!";
 		&'add_log("WARNING cannot open compspec file $cf'compspec: $err")
